#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <ashd/utils.h>
#include <ashd/mtio.h>
#include <ashd/log.h>

#include "statserve.h"

struct fssrc {
    char *path;
};

static struct fileinfo fsserve(struct source *src, char *nm)
{
    struct fssrc *d = src->pdata;
    struct stat sb;
    char *p;
    struct charbuf rb;
    int fd, ret;
    
    if((nm[0] == '.') || strchr(nm, '/'))
	return((struct fileinfo){});
    p = sprintf2("%s/%s", d->path, nm);
    if(stat(p, &sb)) {
	if(errno != ENOENT)
	    flog(LOG_WARNING, "fssrc: %s: %s", p, strerror(errno));
	free(p);
	return((struct fileinfo){});
    }
    if(!S_ISREG(sb.st_mode)) {
	free(p);
	return((struct fileinfo){});
    }
    if((fd = open(p, O_RDONLY)) < 0) {
	flog(LOG_WARNING, "fssrc: %s: %s", p, strerror(errno));
	free(p);
	return((struct fileinfo){});
    }
    bufinit(rb);
    sizebuf(rb, sb.st_size);
    while(1) {
	sizebuf(rb, 4096);
	if((ret = read(fd, rb.b + rb.d, rb.s - rb.d)) < 0) {
	    flog(LOG_ERR, "fssrc: %s: %s", p, strerror(errno));
	    free(p);
	    buffree(rb);
	    close(fd);
	    return((struct fileinfo){});
	}
	if(ret == 0)
	    break;
	rb.d += ret;
    }
    close(fd);
    return((struct fileinfo){.mtime = sb.st_mtime, .sz = rb.d, .data = rb.b, .ctype = "image/png"});
}

struct source *mkfssrc(char *path)
{
    struct source *src;
    struct fssrc *d;
    
    omalloc(src);
    src->serve = fsserve;
    src->pdata = omalloc(d);
    d->path = sstrdup(path);
    return(src);
}
